@@ -3,7 +3,6 @@
 "Date Widget"
 
 import gobject
-import pango
 import gtk
 import re
 import time
@@ -29,6 +28,7 @@ MAPPING = {
     '%M': ('__', '([_ 0-9][_ 0-9])'),
     '%S': ('__', '([_ 0-9][_ 0-9])'),
     '%p': ('__', '([_ AP][_ M])'),
+    '%f': ('______', '([_0-9][_0-9][_0-9][_0-9][_0-9][_0-9])'),
     }
 OPERATORS = {
     gtk.keysyms.S: relativedelta(seconds=-1),
@@ -52,7 +52,6 @@ class DateEntry(gtk.Entry):
 
     def __init__(self, format):
         super(DateEntry, self).__init__()
-        self.modify_font(pango.FontDescription("monospace"))
 
         self.set_format(format)
 
@@ -63,11 +62,10 @@ class DateEntry(gtk.Entry):
         self.connect('focus-in-event', self._focus_in)
         self.connect('focus-out-event', self._focus_out)
 
-        if hasattr(self, 'set_icon_from_stock'):
-            self.set_icon_from_stock(gtk.ENTRY_ICON_SECONDARY, 'tryton-find')
-            self.set_icon_tooltip_text(gtk.ENTRY_ICON_SECONDARY,
-                _('Open the calendar <F2>'))
-            self.connect('icon-press', DateEntry.cal_open)
+        self.set_icon_from_stock(gtk.ENTRY_ICON_SECONDARY, 'tryton-find')
+        self.set_icon_tooltip_text(gtk.ENTRY_ICON_SECONDARY,
+            _('Open the calendar <F2>'))
+        self.connect('icon-press', DateEntry.cal_open)
 
         self._interactive_input = True
         self.idle_set_position(0)
@@ -82,7 +80,7 @@ class DateEntry(gtk.Entry):
         self.regex = re.compile(self.regex)
 
         assert self.regex.match(self.initial_value), \
-                'Error, the initial value should be validated by regex'
+            'Error, the initial value should be validated by regex'
         self.set_width_chars(len(self.initial_value) + 3)  # space for icon
         self.set_max_length(len(self.initial_value))
 
@@ -156,7 +154,11 @@ class DateEntry(gtk.Entry):
     def set_text(self, text):
         self._interactive_input = False
         try:
+            # Force position as _on_insert_text use it
+            position = self.get_position()
+            self.set_position(0)
             gtk.Entry.set_text(self, text)
+            self.idle_set_position(position)
         finally:
             self._interactive_input = True
 
@@ -215,14 +217,15 @@ class DateEntry(gtk.Entry):
             return False
         return True
 
-    def date_get(self):
+    def date_get(self, set_text=True):
         res = None
         date = self.compute_date(self.get_text())
         try:
-            res = datetime.datetime(*time.strptime(date, self.format)[:6])
+            res = datetime.datetime.strptime(date, self.format)
         except ValueError:
             return None
-        self.set_text(date)
+        if set_text:
+            self.set_text(date)
         return res
 
     def delete_text(self, start, end):
@@ -266,6 +269,8 @@ class DateEntry(gtk.Entry):
         return False
 
     def cal_open(self, *args):
+        if not self.get_editable():
+            return False
         parent = self.get_toplevel()
         dialog = gtk.Dialog(_('Date Selection'), parent,
             gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
