@@ -8,7 +8,10 @@ import datetime
 
 def in_(a, b):
     if isinstance(a, (list, tuple)):
-        return any(operator.contains(b, x) for x in a)
+        if isinstance(b, (list, tuple)):
+            return any(operator.contains(b, x) for x in a)
+        else:
+            return operator.contains(a, b)
     else:
         return operator.contains(b, a)
 
@@ -20,7 +23,7 @@ OPERATORS = {
     '>=': operator.ge,
     '!=': operator.ne,
     'in': in_,
-    'not in': lambda a, b: not in_(b, a),
+    'not in': lambda a, b: not in_(a, b),
     # Those operators are not supported (yet ?)
     'like': lambda a, b: True,
     'ilike': lambda a, b: True,
@@ -31,9 +34,9 @@ OPERATORS = {
 }
 
 
-def locale_part(expression, field_name):
+def locale_part(expression, field_name, locale_name='id'):
     if expression == field_name:
-        return 'id'
+        return locale_name
     if '.' in expression:
         fieldname, local = expression.split('.', 1)
         return local
@@ -47,16 +50,21 @@ def is_leaf(expression):
         and expression[1] in OPERATORS)
 
 
+def constrained_leaf(part, boolop=operator.and_):
+    field, operand, value = part[:3]
+    if operand == '=' and boolop == operator.and_:
+        # We should consider that other domain inversion will set a correct
+        # value to this field
+        return True
+    return False
+
+
 def eval_leaf(part, context, boolop=operator.and_):
     field, operand, value = part[:3]
     if '.' in field:
         # In the case where the leaf concerns a m2o then having a value in the
         # evaluation context is deemed suffisant
         return bool(context.get(field.split('.')[0]))
-    if operand == '=' and not context.get(field) and boolop == operator.and_:
-        # We should consider that other domain inversion will set a correct
-        # value to this field
-        return True
     context_field = context.get(field)
     if isinstance(context_field, datetime.date) and not value:
         if isinstance(context_field, datetime.datetime):
@@ -80,6 +88,13 @@ def eval_leaf(part, context, boolop=operator.and_):
             context_field = '%s,%s' % context_field
         except TypeError:
             pass
+    if (operand in ('=', '!=')
+            and isinstance(context_field, (list, tuple))
+            and isinstance(value, (int, long))):
+        operand = {
+            '=': 'in',
+            '!=': 'not in',
+            }[operand]
     return OPERATORS[operand](context_field, value)
 
 
@@ -124,26 +139,23 @@ def localize_domain(domain, field_name=None):
                 return domain
             else:
                 return [domain[3]] + list(domain[1:-1])
-        return [locale_part(domain[0], field_name)] + list(domain[1:])
+        locale_name = 'id'
+        if isinstance(domain[2], basestring):
+            locale_name = 'rec_name'
+        return [locale_part(domain[0], field_name, locale_name)] \
+            + list(domain[1:3]) + list(domain[4:])
     else:
         return [localize_domain(part, field_name) for part in domain]
 
 
-def unlocalize_domain(domain, fieldname):
-    if domain in ('AND', 'OR', True, False):
-        return domain
-    elif is_leaf(domain):
-        return ['%s.%s' % (fieldname, domain[0])] + list(domain[1:])
-    else:
-        return [unlocalize_domain(part, fieldname) for part in domain]
-
-
 def simplify(domain):
     "remove unused domain delimiter"
     if is_leaf(domain):
         return domain
     elif domain in ('OR', 'AND'):
         return domain
+    elif domain in (['OR'], ['AND']):
+        return []
     elif (isinstance(domain, list) and len(domain) == 1
             and not is_leaf(domain[0])):
         return simplify(domain[0])
@@ -170,6 +182,17 @@ def merge(domain, domoperator=None):
         return [merge(domain)]
 
 
+def concat(*domains, **kwargs):
+    domoperator = kwargs.get('domoperator')
+    result = []
+    if domoperator:
+        result.append(domoperator)
+    for domain in domains:
+        if domain:
+            result.append(domain)
+    return simplify(merge(result))
+
+
 def parse(domain):
     if is_leaf(domain):
         return domain
@@ -227,7 +250,8 @@ class And(object):
                 field = part[0]
                 if (field not in context
                         or field in context
-                        and eval_leaf(part, context, operator.and_)):
+                        and (eval_leaf(part, context, operator.and_)
+                            or constrained_leaf(part, operator.and_))):
                     result.append(True)
                 else:
                     return False
@@ -270,7 +294,8 @@ class Or(And):
                 field = part[0]
                 field = self.base(field)
                 if (field in context
-                        and eval_leaf(part, context, operator.or_)):
+                        and (eval_leaf(part, context, operator.or_)
+                                or constrained_leaf(part, operator.or_))):
                     return True
                 elif (field in context
                         and not eval_leaf(part, context, operator.or_)):
@@ -307,7 +332,7 @@ def test_and_inversion():
     domain = [['x', '=', 3], ['y', '=', 5]]
     assert domain_inversion(domain, 'z') is True
     assert domain_inversion(domain, 'z', {'x': 2, 'y': 7}) is True
-    assert domain_inversion(domain, 'x', {'y': False}) == [['x', '=', 3]]
+    assert domain_inversion(domain, 'x', {'y': None}) == [['x', '=', 3]]
 
     domain = [['x.id', '>', 5], ['y', '<', 3]]
     assert domain_inversion(domain, 'y') == [['y', '<', 3]]
@@ -327,7 +352,7 @@ def test_or_inversion():
     assert domain_inversion(domain, 'x', {'y': 4, 'z': 'abc'}) is True
 
     domain = ['OR', ['x', '=', 3], ['y', '=', 5]]
-    assert domain_inversion(domain, 'x', {'y': False}) == [['x', '=', 3]]
+    assert domain_inversion(domain, 'x', {'y': None}) == [['x', '=', 3]]
 
     domain = ['OR', ['x', '=', 3], ['y', '>', 5]]
     assert domain_inversion(domain, 'z') is True
@@ -363,7 +388,7 @@ def test_andor_inversion():
 def test_andand_inversion():
     domain = [[['x', '=', 4], ['y', '>', 6]], ['z', '=', 3]]
     assert domain_inversion(domain, 'z') == [['z', '=', 3]]
-    assert domain_inversion(domain, 'z', {'x': 5}) is False
+    assert domain_inversion(domain, 'z', {'x': 5}) == [['z', '=', 3]]
     assert domain_inversion(domain, 'z', {'y': 5}) is False
     assert domain_inversion(domain, 'z', {'x': 4, 'y': 7}) == [['z', '=', 3]]
 
@@ -411,6 +436,10 @@ def test_simplify():
     assert simplify(domain) == ['OR', [['x', '=', 3]], [['y', '=', 5]]]
     domain = ['OR', ['x', '=', 3], ['AND', ['y', '=', 5]]]
     assert simplify(domain) == ['OR', ['x', '=', 3], [['y', '=', 5]]]
+    domain = ['AND']
+    assert simplify(domain) == []
+    domain = ['OR']
+    assert simplify(domain) == []
 
 
 def test_merge():
@@ -441,6 +470,17 @@ def test_merge():
         ['d', '=', 5], ['e', '=', 6], ['f', '=', 7]]
 
 
+def test_concat():
+    domain1 = [['a', '=', 1]]
+    domain2 = [['b', '=', 2]]
+    assert concat(domain1, domain2) == ['AND', ['a', '=', 1], ['b', '=', 2]]
+    assert concat([], domain1) == domain1
+    assert concat(domain2, []) == domain2
+    assert concat([], []) == []
+    assert concat(domain1, domain2, domoperator='OR') == [
+        'OR', [['a', '=', 1]], [['b', '=', 2]]]
+
+
 def test_evaldomain():
     domain = [['x', '>', 5]]
     assert eval_domain(domain, {'x': 6})
@@ -462,11 +502,24 @@ def test_evaldomain():
     assert eval_domain(domain, {'x': [3, 4]})
     assert not eval_domain(domain, {'x': [1, 2]})
 
+    domain = [['x', 'not in', [3, 5]]]
+    assert not eval_domain(domain, {'x': 3})
+    assert eval_domain(domain, {'x': 4})
+    assert not eval_domain(domain, {'x': [3]})
+    assert not eval_domain(domain, {'x': [3, 4]})
+    assert eval_domain(domain, {'x': [1, 2]})
+
     domain = ['OR', ['x', '>', 10], ['x', '<', 0]]
     assert eval_domain(domain, {'x': 11})
     assert eval_domain(domain, {'x': -4})
     assert not eval_domain(domain, {'x': 5})
 
+    domain = ['OR', ['x', '>', 0], ['x', '=', None]]
+    assert eval_domain(domain, {'x': 1})
+    assert eval_domain(domain, {'x': None})
+    assert not eval_domain(domain, {'x': -1})
+    assert not eval_domain(domain, {'x': 0})
+
     domain = [['x', '>', 0], ['OR', ['x', '=', 3], ['x', '=', 2]]]
     assert not eval_domain(domain, {'x': 1})
     assert eval_domain(domain, {'x': 3})
@@ -494,6 +547,10 @@ def test_evaldomain():
     assert not eval_domain(domain, {'x': ('test', 2)})
     assert not eval_domain(domain, {'x': 'test,2'})
 
+    domain = [['x', '=', 1]]
+    assert eval_domain(domain, {'x': [1, 2]})
+    assert not eval_domain(domain, {'x': [2]})
+
 
 def test_localize():
     domain = [['x', '=', 5]]
@@ -502,6 +559,10 @@ def test_localize():
     domain = [['x', '=', 5], ['x.code', '=', 7]]
     assert localize_domain(domain, 'x') == [['id', '=', 5], ['code', '=', 7]]
 
+    domain = [['x', 'ilike', 'foo%'], ['x.code', '=', 'test']]
+    assert localize_domain(domain, 'x') == \
+        [['rec_name', 'ilike', 'foo%'], ['code', '=', 'test']]
+
     domain = ['OR', ['AND', ['x', '>', 7], ['x', '<', 15]], ['x.code', '=', 8]]
     assert localize_domain(domain, 'x') == \
         ['OR', ['AND', ['id', '>', 7], ['id', '<', 15]], ['code', '=', 8]]
@@ -512,6 +573,12 @@ def test_localize():
     domain = [['x', 'child_of', [1], 'y']]
     assert localize_domain(domain, 'x') == [['y', 'child_of', [1]]]
 
+    domain = [['x.id', '=', 1, 'y']]
+    assert localize_domain(domain, 'x') == [['id', '=', 1]]
+
+    domain = [['a.b.c', '=', 1, 'y', 'z']]
+    assert localize_domain(domain, 'x') == [['b.c', '=', 1, 'z']]
+
 if __name__ == '__main__':
     test_simple_inversion()
     test_and_inversion()
