@@ -294,7 +294,7 @@ class FloatField(Field):
                 record.expr_eval(
                     self.attrs.get('digits', self.default_digits)),
                 self.default_digits))
-        shift = int(round(math.log(factor, 10)))
+        shift = int(round(math.log(abs(factor), 10)))
         return (digits[0] + shift, digits[1] - shift)
 
     def convert(self, value):
@@ -648,6 +648,9 @@ class O2MField(Field):
         self._connect_value(group)
 
     def set_client(self, record, value, force_change=False):
+        # domain inversion could try to set None as value
+        if value is None:
+            value = []
         # domain inversion could try to set id as value
         if isinstance(value, (int, long)):
             value = [value]
@@ -694,7 +697,7 @@ class O2MField(Field):
                 fields = {}
 
         to_remove = []
-        for record2 in record.value[self.name]:
+        for record2 in record.value.get(self.name, []):
             if not record2.id:
                 to_remove.append(record2)
         if value and value.get('remove'):
@@ -905,6 +908,9 @@ class DictField(Field):
 
     _default = {}
 
+    def get(self, record):
+        return super(DictField, self).get(record) or self._default
+
     def get_client(self, record):
         return super(DictField, self).get_client(record) or self._default
 
