@@ -9,13 +9,10 @@ except ImportError:
     import json
 import locale
 from interface import ParserView
-from tryton.action import Action
-from tryton.common import message
 import gettext
 from tryton.config import CONFIG
 from tryton.common.cellrendererbutton import CellRendererButton
 from tryton.common.cellrenderertoggle import CellRendererToggle
-from tryton.pyson import PYSONEncoder
 from tryton.gui.window import Window
 from tryton.common.popup_menu import populate
 from tryton.common import RPCExecute, RPCException
@@ -42,8 +39,6 @@ class AdaptModelGroup(gtk.GenericTreeModel):
     def __init__(self, group, children_field=None):
         super(AdaptModelGroup, self).__init__()
         self.group = group
-        self.last_sort = None
-        self.sort_asc = True
         self.set_property('leak_references', False)
         self.children_field = children_field
         self.__removed = None  # XXX dirty hack to allow update of has_child
@@ -63,9 +58,6 @@ class AdaptModelGroup(gtk.GenericTreeModel):
                 iter_ = self.get_iter(path)
                 self.row_has_child_toggled(path, iter_)
 
-    def cancel(self):
-        pass
-
     def removed(self, group, record):
         if (group is self.group
                 and (record.group is self.group
@@ -99,7 +91,7 @@ class AdaptModelGroup(gtk.GenericTreeModel):
             group.add(record)
             if not record.parent_name:
                 record.modified_fields.setdefault(prev_group.parent_name)
-                record.value[prev_group.parent_name] = False
+                record.value[prev_group.parent_name] = None
             else:
                 record.modified_fields.setdefault(record.parent_name)
         group.move(record, pos)
@@ -119,7 +111,8 @@ class AdaptModelGroup(gtk.GenericTreeModel):
             # Don't remove record from previous group
             # as the new parent will change the parent
             # This prevents concurrency conflict
-            record.group.record_removed.remove(record)
+            if record in record.group.record_removed:
+                record.group.record_removed.remove(record)
             group.add(record)
             record.modified_fields.setdefault(record.parent_name or 'id')
         group.move(record, 0)
@@ -334,9 +327,17 @@ class ViewList(ParserView):
     def modified(self):
         return False
 
+    @property
+    def editable(self):
+        return bool(getattr(self.widget_tree, 'editable', False))
+
     def get_fields(self):
         return [col.name for col in self.widget_tree.get_columns() if col.name]
 
+    def get_buttons(self):
+        return [b for b in self.state_widgets
+            if isinstance(b.renderer, CellRendererButton)]
+
     def on_keypress(self, widget, event):
         if (event.keyval == gtk.keysyms.c
                 and event.state & gtk.gdk.CONTROL_MASK):
@@ -432,7 +433,9 @@ class ViewList(ParserView):
             if not col.get_visible() or not col.name:
                 continue
             cell = self.widget_tree.cells[col.name]
-            values.append('"' + str(cell.get_textual_value(record)) + '"')
+            values.append('"'
+                + str(cell.get_textual_value(record)).replace('"', '""')
+                + '"')
         data.append('\t'.join(values))
         return
 
@@ -448,8 +451,7 @@ class ViewList(ParserView):
         return
 
     def on_paste(self):
-        if (not hasattr(self.widget_tree, 'editable')
-                or not self.widget_tree.editable):
+        if not self.editable:
             return
 
         def unquote(value):
@@ -472,10 +474,21 @@ class ViewList(ParserView):
         if col in columns:
             idx = columns.index(col)
             columns = columns[idx:]
-        record = self.screen.current_record
-        group = record.group
-        idx = group.index(record)
+        if self.screen.current_record:
+            record = self.screen.current_record
+            group = record.group
+            idx = group.index(record)
+        else:
+            group = self.screen.group
+            idx = len(group)
+        default = None
         for line in data:
+            if idx >= len(group):
+                record = group.new(default=False)
+                if default is None:
+                    default = record.default_get()
+                record.set_default(default)
+                group.add(record)
             record = group[idx]
             for col, value in zip(columns, line):
                 cell = self.widget_tree.cells[col.name]
@@ -488,9 +501,6 @@ class ViewList(ParserView):
             if not record.validate():
                 break
             idx += 1
-            if idx >= len(group):
-                # TODO create new record
-                break
         self.screen.current_record = record
         self.screen.display(set_cursor=True)
 
@@ -542,6 +552,15 @@ class ViewList(ParserView):
                     return
         if not selection.data:
             return
+
+        # Don't received if the treeview was editing because it breaks the
+        # internal state of the cursor.
+        cursor, column = treeview.get_cursor()
+        if column:
+            for renderer in column.get_cell_renderers():
+                if renderer.props.editing:
+                    return
+
         store = treeview.get_model()
         try:
             data = json.loads(selection.data)
@@ -586,16 +605,19 @@ class ViewList(ParserView):
 
     def __button_press(self, treeview, event):
         if event.button == 3:
-            path = treeview.get_path_at_pos(int(event.x), int(event.y))
+            try:
+                path, col, x, y = treeview.get_path_at_pos(
+                    int(event.x), int(event.y))
+            except TypeError:
+                # Outside row
+                return False
             selection = treeview.get_selection()
             if selection.get_mode() == gtk.SELECTION_SINGLE:
                 model = selection.get_selected()[0]
             elif selection.get_mode() == gtk.SELECTION_MULTIPLE:
                 model = selection.get_selected_rows()[0]
-            if (not path) or not path[0]:
-                return False
-            group = model.group
-            record = group[path[0][0]]
+            record = model.get_value(model.get_iter(path), 0)
+            group = record.group
             menu = gtk.Menu()
             menu.popup(None, None, None, event.button, event.time)
 
@@ -630,7 +652,7 @@ class ViewList(ParserView):
                     if not model:
                         continue
                     label = field.attrs['string']
-                    populate(menu, model, record_id, title=label)
+                    populate(menu, model, record_id, title=label, field=field)
                 menu.show_all()
             # Delay filling of popup as it can take time
             gobject.idle_add(pop, menu, group, record)
@@ -641,35 +663,6 @@ class ViewList(ParserView):
             return True
         return False
 
-    def click_and_relate(self, action, value, path):
-        data = {}
-        context = {}
-        act = action.copy()
-        if not(value):
-            message(_('You must select a record to use the relation!'))
-            return False
-        from tryton.gui.window.view_form.screen import Screen
-        screen = Screen(self.screen.group.fields[
-            path[1].name].attrs['relation'])
-        screen.load([value])
-        encoder = PYSONEncoder()
-        act['domain'] = encoder.encode(screen.current_record.expr_eval(
-            act.get('domain', [])))
-        act['context'] = encoder.encode(screen.current_record.expr_eval(
-            act.get('context', {})))
-        data['model'] = self.screen.model_name
-        data['id'] = value
-        data['ids'] = [value]
-        return Action._exec_action(act, data, context)
-
-    def click_and_action(self, atype, value, path):
-        return Action.exec_keyword(atype, {
-            'model': self.screen.group.fields[
-                path[1].name].attrs['relation'],
-            'id': value or False,
-            'ids': [value],
-            }, alwaysask=True)
-
     def group_list_changed(self, group, signal):
         if self.store is not None:
             if signal[0] == 'record-added':
@@ -678,9 +671,6 @@ class ViewList(ParserView):
                 self.store.removed(group, signal[1])
         self.display()
 
-    def cancel(self):
-        pass
-
     def __str__(self):
         return 'ViewList (%d)' % id(self)
 
@@ -717,9 +707,6 @@ class ViewList(ParserView):
     def destroy(self):
         self.save_width_height()
         self.widget_tree.destroy()
-        self.screen = None
-        self.widget_tree = None
-        self.widget = None
 
     def __sig_switch(self, treeview, path, column):
         if column._type == 'button':
@@ -752,15 +739,12 @@ class ViewList(ParserView):
         elif tree_sel.get_mode() == gtk.SELECTION_MULTIPLE:
             model, paths = tree_sel.get_selected_rows()
             if model and paths:
-                iter_ = model.get_iter(paths[0])
-                record = model.get_value(iter_, 0)
+                record = model.on_get_iter(paths[0])
                 self.screen.current_record = record
             else:
                 self.screen.current_record = None
 
-        if (hasattr(self.widget_tree, 'editable')
-                and self.widget_tree.editable
-                and previous_record):
+        if self.editable and previous_record:
             def go_previous():
                 self.screen.current_record = previous_record
                 self.set_cursor()
@@ -789,8 +773,7 @@ class ViewList(ParserView):
         self.update_children()
 
     def set_value(self):
-        if hasattr(self.widget_tree, 'editable') \
-                and self.widget_tree.editable:
+        if self.editable:
             self.widget_tree.set_value()
 
     def reset(self):
@@ -799,6 +782,7 @@ class ViewList(ParserView):
     # self.widget.set_model(self.store) could be removed if the store
     # has not changed -> better ergonomy. To test
     def display(self):
+        current_record = self.screen.current_record
         if (self.reload
                 or not self.widget_tree.get_model()
                 or (self.screen.group !=
@@ -806,15 +790,18 @@ class ViewList(ParserView):
             self.store = AdaptModelGroup(self.screen.group,
                     self.children_field)
             self.widget_tree.set_model(self.store)
+            # __select_changed resets current_record to None
+            self.screen.current_record = current_record
+            if current_record:
+                selection = self.widget_tree.get_selection()
+                path = self.store.on_get_path(current_record)
+                selection.select_path(path)
         self.reload = False
-        if not self.screen.current_record:
-            # Should find a simpler solution to do something like
-            #self.widget.set_cursor(None,None,False)
-            if self.store:
-                self.widget_tree.set_model(self.store)
+        if not current_record:
+            selection = self.widget_tree.get_selection()
+            selection.unselect_all()
         self.widget_tree.queue_draw()
-        if hasattr(self.widget_tree, 'editable') \
-                and self.widget_tree.editable:
+        if self.editable:
             self.set_state()
         self.update_children()
 
@@ -827,7 +814,7 @@ class ViewList(ParserView):
                     field.state_set(record)
 
     def update_children(self):
-        ids = self.sel_ids_get()
+        selected_records = self.selected_records
         for child in self.children:
             value = 0
             value_selected = 0
@@ -840,7 +827,7 @@ class ViewList(ParserView):
                 field_value = record.fields_get()[child_fieldname].get(record)
                 if field_value is not None:
                     value += field_value
-                    if record.id in ids or not ids:
+                    if record in selected_records or not selected_records:
                         value_selected += field_value
 
             if loaded:
@@ -859,24 +846,18 @@ class ViewList(ParserView):
             path = self.store.on_get_path(self.screen.current_record)
             if self.store.get_flags() & gtk.TREE_MODEL_LIST_ONLY:
                 path = (path[0],)
-            focus_column = self.widget_tree.next_column(path)
+            focus_column = self.widget_tree.next_column(path, editable=new)
             if path[:-1]:
                 self.widget_tree.expand_to_path(path[:-1])
             self.widget_tree.scroll_to_cell(path, focus_column,
                 use_align=False)
-            self.widget_tree.set_cursor(path, focus_column, new)
-
-    def sel_ids_get(self):
-        def _func_sel_get(store, path, iter, ids):
-            record = store.on_get_iter(path)
-            if record and record.id >= 0:
-                ids.append(record.id)
-        ids = []
-        sel = self.widget_tree.get_selection()
-        if sel:
-            sel.selected_foreach(_func_sel_get, ids)
-        return ids
+            current_path = self.widget_tree.get_cursor()[0]
+            selected_path = \
+                self.widget_tree.get_selection().get_selected_rows()[1]
+            if (current_path != path and path not in selected_path) or new:
+                self.widget_tree.set_cursor(path, focus_column, new)
 
+    @property
     def selected_records(self):
         def _func_sel_get(store, path, iter, records):
             records.append(store.on_get_iter(path))
@@ -898,6 +879,33 @@ class ViewList(ParserView):
                 else:
                     renderer.set_property('editable', False)
 
+    def get_selected_paths(self):
+        selection = self.widget_tree.get_selection()
+        model, rows = selection.get_selected_rows()
+        id_paths = []
+        for row in rows:
+            path = ()
+            id_path = []
+            for node in row:
+                path += (node,)
+                id_path.append(model.on_get_iter(path).id)
+            id_paths.append(id_path)
+        return id_paths
+
+    def select_nodes(self, nodes):
+        selection = self.widget_tree.get_selection()
+        if not nodes:
+            return
+        selection.unselect_all()
+        scroll = False
+        for node in nodes:
+            path = path_convert_id2pos(self.store, node)
+            if path:
+                selection.select_path(path)
+                if not scroll:
+                    self.widget_tree.scroll_to_cell(path)
+                    scroll = True
+
     def get_expanded_paths(self, starting_path=None, starting_id_path=None):
         # Use id instead of position
         # because the position may change between load
