@@ -291,7 +291,7 @@ class WinExport(NoModal):
             return
         try:
             lines = RPCExecute('model', 'ir.export.line', 'read',
-                sum((x['export_fields'] for x in exports), []), None,
+                sum((list(x['export_fields']) for x in exports), []), None,
                 context=self.context)
         except RPCException:
             return
